@@ -6,6 +6,18 @@ Source: https://github.com/BoboTiG/python-mss
 import ctypes
 import ctypes.util
 import sys
+from ctypes import (
+    POINTER,
+    Structure,
+    c_double,
+    c_float,
+    c_int32,
+    c_uint64,
+    c_ubyte,
+    c_uint32,
+    c_void_p,
+)
+from platform import mac_ver
 from typing import TYPE_CHECKING
 
 from .base import MSSBase
@@ -22,13 +34,13 @@ __all__ = ("MSS",)
 
 
 def cgfloat():
-    # type: () -> Union[Type[ctypes.c_double], Type[ctypes.c_float]]
+    # type: () -> Union[Type[c_double], Type[c_float]]
     """ Get the appropriate value for a float. """
 
-    return ctypes.c_double if sys.maxsize > 2 ** 32 else ctypes.c_float
+    return c_double if sys.maxsize > 2 ** 32 else c_float
 
 
-class CGPoint(ctypes.Structure):
+class CGPoint(Structure):
     """ Structure that contains coordinates of a rectangle. """
 
     _fields_ = [("x", cgfloat()), ("y", cgfloat())]
@@ -37,7 +49,7 @@ class CGPoint(ctypes.Structure):
         return "{}(left={} top={})".format(type(self).__name__, self.x, self.y)
 
 
-class CGSize(ctypes.Structure):
+class CGSize(Structure):
     """ Structure that contains dimensions of an rectangle. """
 
     _fields_ = [("width", cgfloat()), ("height", cgfloat())]
@@ -48,7 +60,7 @@ class CGSize(ctypes.Structure):
         )
 
 
-class CGRect(ctypes.Structure):
+class CGRect(Structure):
     """ Structure that contains information about a rectangle. """
 
     _fields_ = [("origin", CGPoint), ("size", CGSize)]
@@ -57,6 +69,42 @@ class CGRect(ctypes.Structure):
         return "{}<{} {}>".format(type(self).__name__, self.origin, self.size)
 
 
+# C functions that will be initialised later.
+#
+# This is a dict:
+#    cfunction: (attr, argtypes, restype)
+#
+# Available attr: core.
+#
+# Note: keep it sorted by cfunction.
+CFUNCTIONS = {
+    "CGDataProviderCopyData": ("core", [c_void_p], c_void_p),
+    "CGDisplayBounds": ("core", [c_uint32], CGRect),
+    "CGDisplayRotation": ("core", [c_uint32], c_float),
+    "CFDataGetBytePtr": ("core", [c_void_p], c_void_p),
+    "CFDataGetLength": ("core", [c_void_p], c_uint64),
+    "CFRelease": ("core", [c_void_p], c_void_p),
+    "CGDataProviderRelease": ("core", [c_void_p], c_void_p),
+    "CGGetActiveDisplayList": (
+        "core",
+        [c_uint32, POINTER(c_uint32), POINTER(c_uint32)],
+        c_int32,
+    ),
+    "CGImageGetBitsPerPixel": ("core", [c_void_p], int),
+    "CGImageGetBytesPerRow": ("core", [c_void_p], int),
+    "CGImageGetDataProvider": ("core", [c_void_p], c_void_p),
+    "CGImageGetHeight": ("core", [c_void_p], int),
+    "CGImageGetWidth": ("core", [c_void_p], int),
+    "CGRectStandardize": ("core", [CGRect], CGRect),
+    "CGRectUnion": ("core", [CGRect, CGRect], CGRect),
+    "CGWindowListCreateImage": (
+        "core",
+        [CGRect, c_uint32, c_uint32, c_uint32],
+        c_void_p,
+    ),
+}
+
+
 class MSS(MSSBase):
     """
     Multiple ScreenShots implementation for macOS.
@@ -72,54 +120,37 @@ class MSS(MSSBase):
 
         self.max_displays = 32
 
-        coregraphics = ctypes.util.find_library("CoreGraphics")
+        self._init_library()
+        self._set_cfunctions()
+
+    def _init_library(self):
+        """ Load the CoreGraphics library. """
+        version = float(".".join(mac_ver()[0].split(".")[:2]))
+        if version < 10.16:
+            coregraphics = ctypes.util.find_library("CoreGraphics")
+        else:
+            # macOS Big Sur and newer
+            # pylint: disable=line-too-long
+            coregraphics = "/System/Library/Frameworks/CoreGraphics.framework/Versions/Current/CoreGraphics"
+
         if not coregraphics:
             raise ScreenShotError("No CoreGraphics library found.")
         self.core = ctypes.cdll.LoadLibrary(coregraphics)
 
-        self._set_cfunctions()
-
     def _set_cfunctions(self):
         # type: () -> None
         """ Set all ctypes functions and attach them to attributes. """
 
-        def cfactory(func, argtypes, restype):
-            # type: (str, List[Any], Any) -> None
-            """ Factorize ctypes creations. """
-            self._cfactory(
-                attr=self.core, func=func, argtypes=argtypes, restype=restype
+        cfactory = self._cfactory
+        attrs = {"core": self.core}
+        for func, (attr, argtypes, restype) in CFUNCTIONS.items():
+            cfactory(
+                attr=attrs[attr],
+                func=func,
+                argtypes=argtypes,  # type: ignore
+                restype=restype,
             )
 
-        uint32 = ctypes.c_uint32
-        void = ctypes.c_void_p
-        size_t = ctypes.c_size_t
-        pointer = ctypes.POINTER
-
-        cfactory(
-            func="CGGetActiveDisplayList",
-            argtypes=[uint32, pointer(uint32), pointer(uint32)],
-            restype=ctypes.c_int32,
-        )
-        cfactory(func="CGDisplayBounds", argtypes=[uint32], restype=CGRect)
-        cfactory(func="CGRectStandardize", argtypes=[CGRect], restype=CGRect)
-        cfactory(func="CGRectUnion", argtypes=[CGRect, CGRect], restype=CGRect)
-        cfactory(func="CGDisplayRotation", argtypes=[uint32], restype=ctypes.c_float)
-        cfactory(
-            func="CGWindowListCreateImage",
-            argtypes=[CGRect, uint32, uint32, uint32],
-            restype=void,
-        )
-        cfactory(func="CGImageGetWidth", argtypes=[void], restype=size_t)
-        cfactory(func="CGImageGetHeight", argtypes=[void], restype=size_t)
-        cfactory(func="CGImageGetDataProvider", argtypes=[void], restype=void)
-        cfactory(func="CGDataProviderCopyData", argtypes=[void], restype=void)
-        cfactory(func="CFDataGetBytePtr", argtypes=[void], restype=void)
-        cfactory(func="CFDataGetLength", argtypes=[void], restype=ctypes.c_uint64)
-        cfactory(func="CGImageGetBytesPerRow", argtypes=[void], restype=size_t)
-        cfactory(func="CGImageGetBitsPerPixel", argtypes=[void], restype=size_t)
-        cfactory(func="CGDataProviderRelease", argtypes=[void], restype=void)
-        cfactory(func="CFRelease", argtypes=[void], restype=void)
-
     def _monitors_impl(self):
         # type: () -> None
         """ Get positions of monitors. It will populate self._monitors. """
@@ -134,8 +165,8 @@ class MSS(MSSBase):
         self._monitors.append({})
 
         # Each monitors
-        display_count = ctypes.c_uint32(0)
-        active_displays = (ctypes.c_uint32 * self.max_displays)()
+        display_count = c_uint32(0)
+        active_displays = (c_uint32 * self.max_displays)()
         core.CGGetActiveDisplayList(
             self.max_displays, active_displays, ctypes.byref(display_count)
         )
@@ -183,20 +214,20 @@ class MSS(MSSBase):
         if not image_ref:
             raise ScreenShotError("CoreGraphics.CGWindowListCreateImage() failed.")
 
-        width = int(core.CGImageGetWidth(image_ref))
-        height = int(core.CGImageGetHeight(image_ref))
+        width = core.CGImageGetWidth(image_ref)
+        height = core.CGImageGetHeight(image_ref)
         prov = copy_data = None
         try:
             prov = core.CGImageGetDataProvider(image_ref)
             copy_data = core.CGDataProviderCopyData(prov)
             data_ref = core.CFDataGetBytePtr(copy_data)
             buf_len = core.CFDataGetLength(copy_data)
-            raw = ctypes.cast(data_ref, ctypes.POINTER(ctypes.c_ubyte * buf_len))
+            raw = ctypes.cast(data_ref, POINTER(c_ubyte * buf_len))
             data = bytearray(raw.contents)
 
             # Remove padding per row
-            bytes_per_row = int(core.CGImageGetBytesPerRow(image_ref))
-            bytes_per_pixel = int(core.CGImageGetBitsPerPixel(image_ref))
+            bytes_per_row = core.CGImageGetBytesPerRow(image_ref)
+            bytes_per_pixel = core.CGImageGetBitsPerPixel(image_ref)
             bytes_per_pixel = (bytes_per_pixel + 7) // 8
 
             if bytes_per_pixel * width != bytes_per_row:
